package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.prediction.service.IThirdDataAccessCloudService;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.*;
import pps.core.prediction.mapper.*;
import pps.core.prediction.service.data.third_active_power.GetThirdActivePowerViewOutput;
import pps.core.prediction.service.data.third_current_well_condition.GetThirdCurrentWellConditionViewOutput;
import pps.core.prediction.service.data.third_photovoltaic_power.GetThirdPhotovoltaicPowerViewOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 第三方数据接入Cloud模块
 *
 * @author ZWT
 * @date 2024/03/19 10:31
 */
@XText("第三方数据接入Cloud模块")
@XService
public class ThirdDataAccessCloudServiceImpl implements IThirdDataAccessCloudService {

    /**
     * 第三方有功功率定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdActivePowerJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_WELL_STATION_PV_DATA,
                new HashMap<>(0));
        List<GetThirdActivePowerViewOutput> outputs = JSON.parseArray(resultStr, GetThirdActivePowerViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        List<ThirdActivePowerView> dtoList = new ArrayList<>(outputs.size());
        List<ThirdActivePowerDailyUpdateView> dtoDailyList = new ArrayList<>(outputs.size());
        DateTime now = DateTime.now();
        DateTime saveDate = DateUtil.beginOfDay(now);
        DateTime saveHour = DateUtil.beginOfHour(now);
        for (GetThirdActivePowerViewOutput o : outputs) {
            dtoList.add(
                    ThirdActivePowerView.builder()
                            .stationName(o.getStationName())
                            .createDate(o.getCreateDate())
                            .photovoltaicPower(o.getPhotovoltaicPower())
                            .meterPower(o.getMeterPower())
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .build()
            );
            dtoDailyList.add(
                    ThirdActivePowerDailyUpdateView.builder()
                            .stationName(o.getStationName())
                            .createDate(o.getCreateDate())
                            .photovoltaicPower(o.getPhotovoltaicPower())
                            .meterPower(o.getMeterPower())
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .saveDate(saveDate)
                            .saveHour(saveHour)
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdActivePowerViewMapper mapper = context.getBean(ThirdActivePowerViewMapper.class);
            //保存历史数据
            mapper.batchInsertList(dtoList);
            ThirdActivePowerDailyUpdateViewMapper dailyMapper = context.getBean(ThirdActivePowerDailyUpdateViewMapper.class);
            //删除当日最新数据
            dailyMapper.deleteByCurrentDayData(dtoDailyList, saveHour);
            //保存当日最新数据
            dailyMapper.batchInsertList(dtoDailyList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方当日油井状态定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdCurrentWellConditionJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_WELL_REAL_PV_DATA,
                new HashMap<>(0));
        List<GetThirdCurrentWellConditionViewOutput> outputs = JSON.parseArray(resultStr, GetThirdCurrentWellConditionViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        //先删后增
        List<ThirdCurrentWellConditionView> dtoList = new ArrayList<>(outputs.size());
        for (GetThirdCurrentWellConditionViewOutput o : outputs) {
            dtoList.add(
                    ThirdCurrentWellConditionView.builder()
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .wellNumber(o.getWellNumber())
                            .nextTime(o.getNextTime())
                            .wellStatus(o.getWellStatus())
                            .cumulativeProduction(o.getCumulativeProduction())
                            .runningTime(o.getRunningTime())
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdCurrentWellConditionViewMapper mapper = context.getBean(ThirdCurrentWellConditionViewMapper.class);
            mapper.deleteByParams(dtoList);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方光伏日发电量定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdPhotovoltaicPowerJob(XContext context) {
        Map<String, Object> param = new HashMap<>(4);
        param.put("Calc_Date", DateUtil.yesterday().toString("yyyy-MM-dd"));
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_STATION_DAILY_PV_DATA,
                param);
        List<GetThirdPhotovoltaicPowerViewOutput> outputs = JSON.parseArray(resultStr, GetThirdPhotovoltaicPowerViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        List<ThirdPhotovoltaicPowerView> dtoList = new ArrayList<>(outputs.size());
        for (GetThirdPhotovoltaicPowerViewOutput output : outputs) {
            dtoList.add(
                    ThirdPhotovoltaicPowerView.builder()
                            .stationName(output.getStationName())
                            .createDate(output.getCreateDate())
                            .photovoltaicPower(output.getPhotovoltaicPower())
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdPhotovoltaicPowerViewMapper mapper = context.getBean(ThirdPhotovoltaicPowerViewMapper.class);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 能耗分析定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult energyConsumptionAnalysisJob(XContext context) {
        DateTime yesterday = DateUtil.yesterday();
        //查昨日光伏发电量
        ThirdPhotovoltaicPowerMapper powerMapper = context.getBean(ThirdPhotovoltaicPowerMapper.class);
        List<ThirdPhotovoltaicPowerEnt> powerList = powerMapper.selectList(new LambdaQueryWrapper<ThirdPhotovoltaicPowerEnt>()
                .eq(ThirdPhotovoltaicPowerEnt::getCreateDate, yesterday)
        );
        if (CollUtil.isEmpty(powerList)) {
            return XServiceResult.OK;
        }
        //通过场站名查线路ID
        Map<String, String> lineMap = this.getBasePowerLineListByNames(context, powerList.stream()
                        .map(ThirdPhotovoltaicPowerEnt::getStationName)
                        .collect(Collectors.toList())).stream()
                .collect(Collectors.toMap(DynamicQueryBasePowerLineOutput::getLineName, DynamicQueryBasePowerLineOutput::getId));
        //查昨日井口耗电量
        WellheadDailyProductionSituationViewMapper mapper = context.getBean(WellheadDailyProductionSituationViewMapper.class);
        List<WellheadDailyProductionSituationView> situationList = mapper.selectSumDailyElectricityConsumption(yesterday);
        Map<String, BigDecimal> collect = situationList.stream()
                .collect(Collectors.toMap(WellheadDailyProductionSituationView::getLineId, WellheadDailyProductionSituationView::getDailyElectricityConsumption));
        //封装
        List<EnergyConsumptionAnalysisView> analysisList = new ArrayList<>(powerList.size());
        for (ThirdPhotovoltaicPowerEnt power : powerList) {
            String lineId = lineMap.get(power.getStationName());
            analysisList.add(EnergyConsumptionAnalysisView.builder()
                    .lineId(lineId)
                    .stationName(power.getStationName())
                    .powerGeneration(power.getPhotovoltaicPower())
                    .powerConsumption(collect.get(lineId))
                    .dataDate(power.getCreateDate())
                    .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                    .build());
        }
        return XTransactionHelper.begin(context, () -> {
            EnergyConsumptionAnalysisViewMapper energyMapper = context.getBean(EnergyConsumptionAnalysisViewMapper.class);
            energyMapper.batchInsertList(analysisList);
            return XServiceResult.OK;
        });
    }

    /*----------------------------------- private -----------------------------------*/

    /**
     * 查询线路列表通过线路名称
     *
     * @param context   上下文
     * @param lineNames ou-id列表
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getBasePowerLineListByNames(XContext context, List<String> lineNames) {
        IBasePowerLineCloudService basePowerLineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineList = basePowerLineCloudService.getBasePowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineNames(lineNames)
                .build());
        lineList.throwIfFail();
        return lineList.getResult();
    }
}
